extern crate glib;
extern crate gio;
extern crate gtk;

use std::env::args;

use gio::prelude::*;
use gtk::prelude::*;

pub struct Async {
    application: gtk::Application,
    progress: gtk::ProgressBar,
}

impl Async {

    pub fn new() -> Async {
        let ui_src = include_str!("application-window.ui");
        let builder = gtk::Builder::new_from_string(ui_src);

        let window: gtk::ApplicationWindow = builder
            .get_object("window").expect("Couldn't get window");
        let progress: gtk::ProgressBar = builder
            .get_object("progress").expect("Couldn't get window");

        window.connect_delete_event(move |win, _| {
            win.destroy();
            Inhibit(false)
        });

        let application = gtk::Application::new("mx.unam.example",
                                                gio::ApplicationFlags::empty())
            .expect("Initialization failed...");

        let win = window.clone();
        application.connect_startup(move |app| {
            win.set_application(app);
        });
        application.connect_activate(|_| {});

        Async {
            application: application,
            progress: progress,
        }
    }

    pub fn run(&self) {
        self.application.run(&args().collect::<Vec<_>>());
    }

    pub fn update(&self, fraction: f64) {
        self.progress.set_fraction(fraction);
    }
}
