extern crate glib;
extern crate gio;
extern crate gtk;

use std::rc::Rc;
use std::thread;
use std::sync::mpsc;
use std::cell::RefCell;
use std::time::Duration;

mod gui;

thread_local!(
    static GLOBAL: RefCell<Option<(Rc<gui::Async>,
                                   mpsc::Receiver<f64>)>> = RefCell::new(None)
);

fn receive() -> glib::Continue {
    GLOBAL.with(|global| {
        if let Some((ref async, ref rx)) = *global.borrow() {
            if let Ok(fraction) = rx.try_recv() {
                async.update(fraction);
            }
        }
    });
    glib::Continue(false)
}

fn main() {
    if gtk::init().is_err() {
        println!("Failed to initialize GTK.");
        return;
    }

    let (tx, rx) = mpsc::channel();

    thread::spawn(move || {
        for i in 1..11 {
            let f = (i as f64) / 10.0;
            println!("hi number {} from the spawned thread!", i);
            tx.send(f).unwrap();
            glib::idle_add(receive);
            thread::sleep(Duration::from_millis(1000));
        }
    });

    let async = Rc::new(gui::Async::new());
    let async_1 = async.clone();

    GLOBAL.with(move |global| {
        *global.borrow_mut() = Some((async_1, rx))
    });

    async.run();
}
